module config_parser

    implicit none
    public

    !< @brief C->Fortran interface 
    interface
        !< process configuration file [filename], status = [0 (ERROR), 1 (OK)]
        subroutine c_config_run(filename, status) BIND(C)
            use, intrinsic :: iso_c_binding, only: C_CHAR, C_INT
            character (kind=C_CHAR), intent(in) :: filename(*)
            integer (kind=C_INT), intent(out) :: status
        end subroutine c_config_run

        !< check if variable [name] is defined, status = [0 (FALSE), 1 (TRUE)]
        subroutine c_config_is_varname(name, status) BIND(C)
            use, intrinsic :: iso_c_binding, only: C_CHAR, C_INT
            character (kind=C_CHAR), intent(in) :: name(*)
            integer (kind=C_INT), intent(out) :: status
        end subroutine c_config_is_varname

        !< get (int) [value] of variable [name], status = [0 (ERROR), 1 (OK)]
        subroutine c_config_get_int(name, value, status) BIND(C)
            use, intrinsic :: iso_c_binding, only: C_CHAR, C_INT
            character (kind=C_CHAR), intent(in) :: name(*)
            integer (kind=C_INT), intent(out) :: value
            integer (kind=C_INT), intent(out) :: status
        end subroutine c_config_get_int

        !< get (float) [value] of variable [name], status = [0 (ERROR), 1 (OK)]
        subroutine c_config_get_float(name, value, status) BIND(C)
            use, intrinsic :: iso_c_binding, only: C_CHAR, C_INT, C_FLOAT
            character (kind=C_CHAR), intent(in) :: name(*)
            real (kind=C_FLOAT), intent(out) :: value
            integer (kind=C_INT), intent(out) :: status
        end subroutine c_config_get_float

        !< get (double) [value] of variable [name], status = [0 (ERROR), 1 (OK)]
        subroutine c_config_get_double(name, value, status) BIND(C)
            use, intrinsic :: iso_c_binding, only: C_CHAR, C_INT, C_DOUBLE
            character (kind=C_CHAR), intent(in) :: name(*)
            real (kind=C_DOUBLE), intent(out) :: value
            integer (kind=C_INT), intent(out) :: status
        end subroutine c_config_get_double

        !< get (int) [len] of string variable [name]
        !<      [len] = 0 if variable is not defined or not a string
        subroutine c_config_get_string_len(name, len) BIND(C)
            use, intrinsic :: iso_c_binding, only: C_CHAR, C_INT
            character (kind=C_CHAR), intent(in) :: name(*)
            integer (kind=C_INT), intent(out) :: len
        end subroutine c_config_get_string_len

        !< get (char*) [c_str] of string variable [name], status = [0 (ERROR), 1 (OK)]
        !<       *: [c_str] should be allocated up to hold at least [len] bytes
        !<       *: [c_str] output doesn't contain the null terminated character
        subroutine c_config_get_string_unsafe(name, c_str, status) BIND(C)
            use, intrinsic :: iso_c_binding, only: C_CHAR, C_INT
            character (kind=C_CHAR), intent(in) :: name(*)
            character (kind=C_CHAR), intent(out) :: c_str(*)
            integer (kind=C_INT), intent(out) :: status
        end subroutine c_config_get_string_unsafe

    end interface 

    contains

    !< @brief additional C->Fortran wrappers
    ! --------------------------------------------------------------------------------
    
    !< get (char*) [c_str] of string variable [name], status = [0 (ERROR), 1 (OK)]
    !       *: [c_str] allocation is performed in-place
    !       *: [c_str] output doesn't contain the null terminated character
    subroutine c_config_get_string(name, c_str, status)
        use, intrinsic :: iso_c_binding, only: C_CHAR, C_INT
        character (kind=C_CHAR), intent(in) :: name(*)
        character (kind=C_CHAR), allocatable, intent(out) :: c_str(:)
        integer (kind=C_INT), intent(out) :: status

        ! local variables
        ! --------------------------------------------------------------------------------
        integer :: len
        ! --------------------------------------------------------------------------------

        call c_config_get_string_len(name, len)
        
        if (allocated(c_str)) deallocate(c_str)
        if (len > 0) allocate(c_str(len))

        call c_config_get_string_unsafe(name, c_str, status)
    end subroutine c_config_get_string
    ! --------------------------------------------------------------------------------
    
    !< @brief Fortran nice wrappers 
    ! --------------------------------------------------------------------------------
    ! --------------------------------------------------------------------------------

end module config_parser
